from typing import Optional, List
from enum import Enum
from pydantic import BaseModel, Field


class MarzbanNodeStatus(str, Enum):
    """Node status enumeration"""

    CONNECTED = "connected"
    CONNECTING = "connecting"
    ERROR = "error"
    DISABLED = "disabled"


class MarzbanNodeCreate(BaseModel):
    """Schema for creating a new node"""

    name: str = Field(..., title="Name")
    address: str = Field(..., title="Address")
    port: int = Field(62050, title="Port")
    api_port: int = Field(62051, title="Api Port")
    usage_coefficient: float = Field(1.0, title="Usage Coefficient", gt=0.0)
    add_as_new_host: bool = Field(True, title="Add As New Host")


class MarzbanNodeModify(BaseModel):
    """Schema for modifying an existing node"""

    name: Optional[str] = Field(None, title="Name")
    address: Optional[str] = Field(None, title="Address")
    port: Optional[int] = Field(None, title="Port")
    api_port: Optional[int] = Field(None, title="Api Port")
    usage_coefficient: Optional[float] = Field(None, title="Usage Coefficient")
    status: Optional[MarzbanNodeStatus] = Field(None, title="Status")


class MarzbanNodeResponse(BaseModel):
    """Node response schema"""

    id: int = Field(..., title="Id")
    name: str = Field(..., title="Name")
    address: str = Field(..., title="Address")
    port: int = Field(62050, title="Port")
    api_port: int = Field(62051, title="Api Port")
    usage_coefficient: float = Field(1.0, title="Usage Coefficient", gt=0.0)
    xray_version: Optional[str] = Field(None, title="Xray Version")
    status: MarzbanNodeStatus = Field(..., title="Status")
    message: Optional[str] = Field(None, title="Message")


class MarzbanNodeSettings(BaseModel):
    """Node settings schema"""

    min_node_version: str = Field("v0.2.0", title="Min Node Version")
    certificate: str = Field(..., title="Certificate")


class MarzbanNodeUsageResponse(BaseModel):
    """Node usage response schema"""

    node_id: Optional[int] = Field(None, title="Node Id")
    node_name: str = Field(..., title="Node Name")
    uplink: int = Field(..., title="Uplink")
    downlink: int = Field(..., title="Downlink")


class MarzbanNodesUsageResponse(BaseModel):
    """Multiple nodes usage response schema"""

    usages: List[MarzbanNodeUsageResponse] = Field(..., title="Usages")



from .admin import OVPanelAdmin, OVPanelToken
from .user import (
    OVPanelUser,
    OVPanelCreateUser,
    OVPanelUpdateUser,
    OVPanelResponseModel,
)
from .node import OVPanelNode, OVPanelNodeCreate, OVPanelNodeStatus
from .system import OVPanelSettings, OVPanelServerInfo

__all__ = [
    "OVPanelAdmin",
    "OVPanelToken",
    "OVPanelUser",
    "OVPanelCreateUser",
    "OVPanelUpdateUser",
    "OVPanelResponseModel",
    "OVPanelNode",
    "OVPanelNodeCreate",
    "OVPanelNodeStatus",
    "OVPanelSettings",
    "OVPanelServerInfo",
]



from pydantic import BaseModel, Field


class MarzbanCoreStats(BaseModel):
    """Core statistics schema"""

    version: str = Field(..., title="Version")
    started: bool = Field(..., title="Started")
    logs_websocket: str = Field(..., title="Logs Websocket")



from typing import Optional, Any
from pydantic import BaseModel, Field


class OVPanelSettings(BaseModel):
    """Panel settings response schema"""

    success: bool = Field(..., title="Success")
    msg: str = Field(..., title="Message")
    data: Optional[Any] = Field(None, title="Data")


class OVPanelServerInfo(BaseModel):
    """Server information response schema"""

    success: bool = Field(..., title="Success")
    msg: str = Field(..., title="Message")
    data: Optional[dict] = Field(None, title="Data")



from enum import Enum
from pydantic import BaseModel, Field


class MarzbanSystemStats(BaseModel):
    """System statistics schema"""

    version: str = Field(..., title="Version")
    mem_total: int = Field(..., title="Mem Total")
    mem_used: int = Field(..., title="Mem Used")
    cpu_cores: int = Field(..., title="Cpu Cores")
    cpu_usage: float = Field(..., title="Cpu Usage")
    total_user: int = Field(..., title="Total User")
    online_users: int = Field(..., title="Online Users")
    users_active: int = Field(..., title="Users Active")
    users_on_hold: int = Field(..., title="Users On Hold")
    users_disabled: int = Field(..., title="Users Disabled")
    users_expired: int = Field(..., title="Users Expired")
    users_limited: int = Field(..., title="Users Limited")
    incoming_bandwidth: int = Field(..., title="Incoming Bandwidth")
    outgoing_bandwidth: int = Field(..., title="Outgoing Bandwidth")
    incoming_bandwidth_speed: int = Field(..., title="Incoming Bandwidth Speed")
    outgoing_bandwidth_speed: int = Field(..., title="Outgoing Bandwidth Speed")


class MarzbanProxyTypes(str, Enum):
    """Proxy types enumeration"""

    VMESS = "vmess"
    VLESS = "vless"
    TROJAN = "trojan"
    SHADOWSOCKS = "shadowsocks"


class MarzbanProxyInbound(BaseModel):
    """Proxy inbound configuration schema"""

    tag: str = Field(..., title="Tag")
    protocol: MarzbanProxyTypes = Field(..., title="Protocol")
    network: str = Field(..., title="Network")
    tls: str = Field(..., title="Tls")
    port: int | str = Field(..., title="Port")


class MarzbanProxyHostSecurity(str, Enum):
    """Proxy host security types"""

    INBOUND_DEFAULT = "inbound_default"
    NONE = "none"
    TLS = "tls"


class MarzbanProxyHostALPN(str, Enum):
    """Proxy host ALPN types"""

    EMPTY = ""
    H3 = "h3"
    H2 = "h2"
    HTTP_1_1 = "http/1.1"
    H3_H2_HTTP_1_1 = "h3,h2,http/1.1"
    H3_H2 = "h3,h2"
    H2_HTTP_1_1 = "h2,http/1.1"


class MarzbanProxyHostFingerprint(str, Enum):
    """Proxy host fingerprint types"""

    EMPTY = ""
    CHROME = "chrome"
    FIREFOX = "firefox"
    SAFARI = "safari"
    IOS = "ios"
    ANDROID = "android"
    EDGE = "edge"
    BROWSER_360 = "360"
    QQ = "qq"
    RANDOM = "random"
    RANDOMIZED = "randomized"


class MarzbanProxyHost(BaseModel):
    """Proxy host configuration schema"""

    remark: str = Field(..., title="Remark")
    address: str = Field(..., title="Address")
    port: int | None = Field(None, title="Port")
    sni: str | None = Field(None, title="Sni")
    host: str | None = Field(None, title="Host")
    path: str | None = Field(None, title="Path")
    security: MarzbanProxyHostSecurity = Field(
        MarzbanProxyHostSecurity.INBOUND_DEFAULT, title="Security"
    )
    alpn: MarzbanProxyHostALPN = Field(MarzbanProxyHostALPN.EMPTY, title="Alpn")
    fingerprint: MarzbanProxyHostFingerprint = Field(
        MarzbanProxyHostFingerprint.EMPTY, title="Fingerprint"
    )
    allowinsecure: bool | None = Field(None, title="Allowinsecure")
    is_disabled: bool | None = Field(None, title="Is Disabled")
    mux_enable: bool | None = Field(None, title="Mux Enable")
    fragment_setting: str | None = Field(None, title="Fragment Setting")
    noise_setting: str | None = Field(None, title="Noise Setting")
    random_user_agent: bool | None = Field(None, title="Random User Agent")
    use_sni_as_host: bool | None = Field(None, title="Use Sni As Host")



from .manager import MarzneshinManager
from .types import (
    MarzneshinAdminCreate,
    MarzneshinAdminPartialModify,
    MarzneshinAdmin,
    MarzneshinAdminResponse,
    MarzneshinToken,
    MarzneshinNodeStatus,
    MarzneshinNodeConnectionBackend,
    MarzneshinNodeCreate,
    MarzneshinNodeModify,
    MarzneshinBackend,
    MarzneshinNodeResponse,
    MarzneshinNodeSettings,
    MarzneshinServiceCreate,
    MarzneshinServiceModify,
    MarzneshinServiceResponse,
    MarzneshinUserExpireStrategy,
    MarzneshinUserDataUsageResetStrategy,
    MarzneshinUsersSortingOptions,
    MarzneshinUserCreate,
    MarzneshinUserModify,
    MarzneshinUserResponse,
    MarzneshinUserNodeUsageSeries,
    MarzneshinUserUsageSeriesResponse,
    MarzneshinConfigTypes,
    MarzneshinSubscriptionRule,
    MarzneshinSubscriptionSettings,
    MarzneshinAdminsStats,
    MarzneshinNodesStats,
    MarzneshinUsersStats,
    MarzneshinTrafficUsageSeries,
    MarzneshinTelegramSettings,
    MarzneshinProxyTypes,
    MarzneshinProxyHostALPN,
    MarzneshinProxyHostFingerprint,
    MarzneshinInboundHostSecurity,
    MarzneshinFragmentSettings,
    MarzneshinXrayNoise,
    MarzneshinXMuxSettings,
    MarzneshinSplitHttpSettings,
    MarzneshinSingBoxMuxSettings,
    MarzneshinMuxCoolSettings,
    MarzneshinMuxSettings,
    MarzneshinInboundHost,
    MarzneshinInboundHostResponse,
    MarzneshinNode,
    MarzneshinInbound,
    MarzneshinBackendConfigFormat,
    MarzneshinBackendConfig,
    MarzneshinBackendStats,
)

__all__ = [
    "MarzneshinManager",
    "MarzneshinAdminCreate",
    "MarzneshinAdminPartialModify",
    "MarzneshinAdmin",
    "MarzneshinAdminResponse",
    "MarzneshinToken",
    "MarzneshinNodeStatus",
    "MarzneshinNodeConnectionBackend",
    "MarzneshinNodeCreate",
    "MarzneshinNodeModify",
    "MarzneshinBackend",
    "MarzneshinNodeResponse",
    "MarzneshinNodeSettings",
    "MarzneshinServiceCreate",
    "MarzneshinServiceModify",
    "MarzneshinServiceResponse",
    "MarzneshinUserExpireStrategy",
    "MarzneshinUserDataUsageResetStrategy",
    "MarzneshinUsersSortingOptions",
    "MarzneshinUserCreate",
    "MarzneshinUserModify",
    "MarzneshinUserResponse",
    "MarzneshinUserNodeUsageSeries",
    "MarzneshinUserUsageSeriesResponse",
    "MarzneshinConfigTypes",
    "MarzneshinSubscriptionRule",
    "MarzneshinSubscriptionSettings",
    "MarzneshinAdminsStats",
    "MarzneshinNodesStats",
    "MarzneshinUsersStats",
    "MarzneshinTrafficUsageSeries",
    "MarzneshinTelegramSettings",
    "MarzneshinProxyTypes",
    "MarzneshinProxyHostALPN",
    "MarzneshinProxyHostFingerprint",
    "MarzneshinInboundHostSecurity",
    "MarzneshinFragmentSettings",
    "MarzneshinXrayNoise",
    "MarzneshinXMuxSettings",
    "MarzneshinSplitHttpSettings",
    "MarzneshinSingBoxMuxSettings",
    "MarzneshinMuxCoolSettings",
    "MarzneshinMuxSettings",
    "MarzneshinInboundHost",
    "MarzneshinInboundHostResponse",
    "MarzneshinNode",
    "MarzneshinInbound",
    "MarzneshinBackendConfigFormat",
    "MarzneshinBackendConfig",
    "MarzneshinBackendStats",
]



from typing import Optional, List, Dict, Any
from pydantic import BaseModel, Field
from datetime import datetime
from enum import Enum


class PasarGuardUserStatus(str, Enum):
    """User status enumeration"""

    active = "active"
    disabled = "disabled"
    limited = "limited"
    expired = "expired"
    on_hold = "on_hold"


class PasarGuardDataLimitResetStrategy(str, Enum):
    """Data limit reset strategy"""

    no_reset = "no_reset"
    day = "day"
    week = "week"
    month = "month"
    year = "year"


class PasarGuardUserCreate(BaseModel):
    """Schema for creating a new user"""

    username: str = Field(..., title="Username")
    proxy_settings: Optional[Dict[str, Any]] = Field(None, title="Proxy Settings")
    expire: Optional[Any] = Field(None, title="Expire")
    data_limit: Optional[int] = Field(None, title="Data Limit")
    data_limit_reset_strategy: Optional[PasarGuardDataLimitResetStrategy] = Field(
        None, title="Data Limit Reset Strategy"
    )
    note: Optional[str] = Field(None, title="Note")
    on_hold_expire_duration: Optional[int] = Field(
        None, title="On Hold Expire Duration"
    )
    on_hold_timeout: Optional[Any] = Field(None, title="On Hold Timeout")
    group_ids: Optional[List[int]] = Field(None, title="Group Ids")
    auto_delete_in_days: Optional[int] = Field(None, title="Auto Delete In Days")
    status: Optional[str] = Field(None, title="Status")


class PasarGuardUserModify(BaseModel):
    """Schema for modifying an existing user"""

    proxy_settings: Optional[Dict[str, Any]] = Field(None, title="Proxy Settings")
    expire: Optional[Any] = Field(None, title="Expire")
    data_limit: Optional[int] = Field(None, title="Data Limit")
    data_limit_reset_strategy: Optional[PasarGuardDataLimitResetStrategy] = Field(
        None, title="Data Limit Reset Strategy"
    )
    note: Optional[str] = Field(None, title="Note")
    on_hold_expire_duration: Optional[int] = Field(
        None, title="On Hold Expire Duration"
    )
    on_hold_timeout: Optional[Any] = Field(None, title="On Hold Timeout")
    group_ids: Optional[List[int]] = Field(None, title="Group Ids")
    auto_delete_in_days: Optional[int] = Field(None, title="Auto Delete In Days")
    status: Optional[str] = Field(None, title="Status")


class PasarGuardUserResponse(BaseModel):
    """User response schema"""

    id: int = Field(..., title="Id")
    username: str = Field(..., title="Username")
    status: PasarGuardUserStatus = Field(..., title="Status")
    used_traffic: int = Field(..., title="Used Traffic")
    lifetime_used_traffic: int = Field(0, title="Lifetime Used Traffic")
    created_at: datetime = Field(..., title="Created At")
    proxy_settings: Optional[Dict[str, Any]] = Field(None, title="Proxy Settings")
    expire: Optional[Any] = Field(None, title="Expire")
    data_limit: Optional[int] = Field(None, title="Data Limit")
    data_limit_reset_strategy: Optional[PasarGuardDataLimitResetStrategy] = Field(
        None, title="Data Limit Reset Strategy"
    )
    note: Optional[str] = Field(None, title="Note")
    on_hold_expire_duration: Optional[int] = Field(
        None, title="On Hold Expire Duration"
    )
    on_hold_timeout: Optional[Any] = Field(None, title="On Hold Timeout")
    group_ids: Optional[List[int]] = Field(None, title="Group Ids")
    auto_delete_in_days: Optional[int] = Field(None, title="Auto Delete In Days")
    edit_at: Optional[datetime] = Field(None, title="Edit At")
    online_at: Optional[datetime] = Field(None, title="Online At")
    subscription_url: str = Field("", title="Subscription Url")
    admin: Optional[Dict[str, Any]] = Field(None, title="Admin")


class PasarGuardUsersResponse(BaseModel):
    """Users list response schema"""

    users: List[PasarGuardUserResponse] = Field(..., title="Users")
    total: int = Field(..., title="Total")


class PasarGuardSubscriptionUserResponse(BaseModel):
    """Subscription user response schema"""

    id: int = Field(..., title="Id")
    username: str = Field(..., title="Username")
    status: PasarGuardUserStatus = Field(..., title="Status")
    used_traffic: int = Field(..., title="Used Traffic")
    lifetime_used_traffic: int = Field(0, title="Lifetime Used Traffic")
    created_at: datetime = Field(..., title="Created At")
    proxy_settings: Optional[Dict[str, Any]] = Field(None, title="Proxy Settings")
    expire: Optional[Any] = Field(None, title="Expire")
    data_limit: Optional[int] = Field(None, title="Data Limit")
    data_limit_reset_strategy: Optional[PasarGuardDataLimitResetStrategy] = Field(
        None, title="Data Limit Reset Strategy"
    )
    on_hold_expire_duration: Optional[int] = Field(
        None, title="On Hold Expire Duration"
    )
    on_hold_timeout: Optional[Any] = Field(None, title="On Hold Timeout")
    group_ids: Optional[List[int]] = Field(None, title="Group Ids")
    edit_at: Optional[datetime] = Field(None, title="Edit At")
    online_at: Optional[datetime] = Field(None, title="Online At")



from typing import Optional
from pydantic import BaseModel, Field


class RemnawaveNode(BaseModel):
    """Node response"""

    uuid: str
    name: str
    address: str
    port: int
    is_connected: bool = Field(..., alias="isConnected")
    is_disabled: bool = Field(..., alias="isDisabled")
    is_node_online: bool = Field(..., alias="isNodeOnline")
    is_xray_running: bool = Field(..., alias="isXrayRunning")
    xray_version: Optional[str] = Field(None, alias="xrayVersion")
    node_version: Optional[str] = Field(None, alias="nodeVersion")
    users_online: Optional[int] = Field(None, alias="usersOnline")
    country_code: str = Field(..., alias="countryCode")
    consumption_multiplier: float = Field(..., alias="consumptionMultiplier")
    traffic_limit_bytes: Optional[int] = Field(None, alias="trafficLimitBytes")
    traffic_used_bytes: Optional[int] = Field(None, alias="trafficUsedBytes")
    created_at: str = Field(..., alias="createdAt")
    updated_at: str = Field(..., alias="updatedAt")

    class Config:
        populate_by_name = True


class RemnawaveNodeCreate(BaseModel):
    """Node creation data"""

    name: str = Field(..., min_length=3, max_length=30)
    address: str = Field(..., min_length=2)
    port: int = Field(..., ge=1, le=65535)
    country_code: str = Field("XX", alias="countryCode", max_length=2)
    consumption_multiplier: float = Field(1.0, alias="consumptionMultiplier", ge=0.1)
    traffic_limit_bytes: int = Field(0, alias="trafficLimitBytes", ge=0)
    is_traffic_tracking_active: bool = Field(False, alias="isTrafficTrackingActive")

    class Config:
        populate_by_name = True


class RemnawaveNodeUpdate(BaseModel):
    """Node update data"""

    name: Optional[str] = Field(None, min_length=3, max_length=30)
    address: Optional[str] = Field(None, min_length=2)
    port: Optional[int] = Field(None, ge=1, le=65535)
    country_code: Optional[str] = Field(None, alias="countryCode", max_length=2)
    consumption_multiplier: Optional[float] = Field(
        None, alias="consumptionMultiplier", ge=0.1
    )
    traffic_limit_bytes: Optional[int] = Field(None, alias="trafficLimitBytes", ge=0)
    is_traffic_tracking_active: Optional[bool] = Field(
        None, alias="isTrafficTrackingActive"
    )

    class Config:
        populate_by_name = True



from typing import Optional, List
from pydantic import BaseModel, Field


class PasarGuardGroupCreate(BaseModel):
    """Schema for creating a new group"""

    name: str = Field(..., title="Name", min_length=3, max_length=64)
    inbound_tags: List[str] = Field(..., title="Inbound Tags")
    is_disabled: bool = Field(False, title="Is Disabled")


class PasarGuardGroupModify(BaseModel):
    """Schema for modifying an existing group"""

    name: str = Field(..., title="Name", min_length=3, max_length=64)
    inbound_tags: Optional[List[str]] = Field([], title="Inbound Tags")
    is_disabled: bool = Field(False, title="Is Disabled")


class PasarGuardGroupResponse(BaseModel):
    """Group response schema"""

    id: int = Field(..., title="Id")
    name: str = Field(..., title="Name", min_length=3, max_length=64)
    inbound_tags: Optional[List[str]] = Field([], title="Inbound Tags")
    is_disabled: bool = Field(False, title="Is Disabled")
    total_users: int = Field(0, title="Total Users")



from .manager import PasarGuardManager
from .types import (
    PasarGuardAdminCreate,
    PasarGuardAdminModify,
    PasarGuardAdminDetails,
    PasarGuardToken,
    PasarGuardUserCreate,
    PasarGuardUserModify,
    PasarGuardUserResponse,
    PasarGuardUsersResponse,
    PasarGuardUserStatus,
    PasarGuardSubscriptionUserResponse,
    PasarGuardNodeCreate,
    PasarGuardNodeModify,
    PasarGuardNodeResponse,
    PasarGuardNodeSettings,
    PasarGuardNodeStatus,
    PasarGuardCoreCreate,
    PasarGuardCoreResponse,
    PasarGuardSystemStats,
    PasarGuardGroupCreate,
    PasarGuardGroupModify,
    PasarGuardGroupResponse,
    PasarGuardHostCreate,
    PasarGuardHostResponse,
    PasarGuardUserTemplateCreate,
    PasarGuardUserTemplateModify,
    PasarGuardUserTemplateResponse,
)

__all__ = [
    "PasarGuardManager",
    "PasarGuardAdminCreate",
    "PasarGuardAdminModify",
    "PasarGuardAdminDetails",
    "PasarGuardToken",
    "PasarGuardUserCreate",
    "PasarGuardUserModify",
    "PasarGuardUserResponse",
    "PasarGuardUsersResponse",
    "PasarGuardUserStatus",
    "PasarGuardSubscriptionUserResponse",
    "PasarGuardNodeCreate",
    "PasarGuardNodeModify",
    "PasarGuardNodeResponse",
    "PasarGuardNodeSettings",
    "PasarGuardNodeStatus",
    "PasarGuardCoreCreate",
    "PasarGuardCoreResponse",
    "PasarGuardSystemStats",
    "PasarGuardGroupCreate",
    "PasarGuardGroupModify",
    "PasarGuardGroupResponse",
    "PasarGuardHostCreate",
    "PasarGuardHostResponse",
    "PasarGuardUserTemplateCreate",
    "PasarGuardUserTemplateModify",
    "PasarGuardUserTemplateResponse",
]



from typing import List, Optional
from datetime import datetime
from pydantic import BaseModel, Field


class GuardSubscriptionCreate(BaseModel):
    """Schema for creating a new subscription"""

    username: str = Field(..., title="Username")
    limit_usage: int = Field(..., title="Limit Usage")
    limit_expire: int = Field(..., title="Limit Expire")
    service_ids: List[int] = Field(..., title="Service Ids")
    access_key: Optional[str] = Field(None, title="Access Key")


class GuardSubscriptionUpdate(BaseModel):
    """Schema for updating an existing subscription"""

    limit_usage: Optional[int] = Field(None, title="Limit Usage")
    limit_expire: Optional[int] = Field(None, title="Limit Expire")
    service_ids: Optional[List[int]] = Field(None, title="Service Ids")


class GuardSubscriptionResponse(BaseModel):
    """Subscription response schema"""

    id: int = Field(..., title="Id")
    username: str = Field(..., title="Username")
    owner_username: str = Field(..., title="Owner Username")
    access_key: str = Field(..., title="Access Key")
    enabled: bool = Field(..., title="Enabled")
    activated: bool = Field(..., title="Activated")
    reached: bool = Field(..., title="Reached")
    limited: bool = Field(..., title="Limited")
    expired: bool = Field(..., title="Expired")
    is_active: bool = Field(..., title="Is Active")
    is_online: bool = Field(..., title="Is Online")
    link: str = Field(..., title="Link")
    limit_usage: int = Field(..., title="Limit Usage")
    reset_usage: int = Field(..., title="Reset Usage")
    total_usage: int = Field(..., title="Total Usage")
    current_usage: int = Field(..., title="Current Usage")
    limit_expire: int = Field(..., title="Limit Expire")
    service_ids: List[int] = Field(..., title="Service Ids")
    online_at: Optional[datetime] = Field(..., title="Online At")
    last_reset_at: Optional[datetime] = Field(..., title="Last Reset At")
    last_revoke_at: Optional[datetime] = Field(..., title="Last Revoke At")
    last_request_at: Optional[datetime] = Field(..., title="Last Request At")
    last_client_agent: Optional[str] = Field(..., title="Last Client Agent")
    created_at: datetime = Field(..., title="Created At")
    updated_at: datetime = Field(..., title="Updated At")


class GuardSubscriptionUsageLog(BaseModel):
    """Subscription usage log entry"""

    usage: int = Field(..., title="Usage")
    created_at: datetime = Field(..., title="Created At")


class GuardSubscriptionUsageLogsResponse(BaseModel):
    """Subscription usage logs response"""

    subscription: GuardSubscriptionResponse = Field(..., title="Subscription")
    usages: List[GuardSubscriptionUsageLog] = Field(..., title="Usages")


class GuardSubscriptionStatsResponse(BaseModel):
    """Subscription statistics response"""

    total: int = Field(..., title="Total")
    active: int = Field(..., title="Active")
    inactive: int = Field(..., title="Inactive")
    disabled: int = Field(..., title="Disabled")
    expired: int = Field(..., title="Expired")
    limited: int = Field(..., title="Limited")
    has_revoked: int = Field(..., title="Has Revoked")
    has_reseted: int = Field(..., title="Has Reseted")
    total_removed: int = Field(..., title="Total Removed")
    total_usage: int = Field(..., title="Total Usage")



from typing import Optional, List, Dict, Any
from opexcore.core import RequestBase
from .types import (
    AdminStatsResponseNew,
    GuardAdminCreate,
    GuardAdminCurrentUpdate,
    GuardAdminResponse,
    GuardAdminToken,
    GuardAdminUpdate,
    GuardAdminUsageLogsResponse,
    GuardNodeCreate,
    GuardNodeResponse,
    GuardNodeStatsResponse,
    GuardNodeUpdate,
    GuardServiceCreate,
    GuardServiceResponse,
    GuardServiceUpdate,
    GuardStatsResponse,
    GuardSubscriptionCreate,
    GuardSubscriptionResponse,
    GuardSubscriptionStatsResponse,
    GuardSubscriptionUpdate,
    GuardSubscriptionUsageLogsResponse,
)


class GuardManager(RequestBase):
    """GuardCore API Manager with all endpoints"""

    @classmethod
    def _generate_headers(cls, token: Optional[str] = None) -> Dict[str, str]:
        headers = {"Content-Type": "application/json"}
        if token:
            headers["Authorization"] = f"Bearer {token}"
        return headers

    @classmethod
    async def get_subscription_by_secret(
        cls, host: str, secret: str, timeout: int = 10
    ) -> List[str]:
        """
        Handle incoming subscription request from clients.

        :param host: API host URL
        :param secret: Subscription secret
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/guards/{secret}", timeout=timeout
        )
        return await response.json()

    @classmethod
    async def get_subscription_info_by_secret(
        cls, host: str, secret: str, timeout: int = 10
    ) -> GuardSubscriptionResponse:
        """
        Get subscription information by secret.

        :param host: API host URL
        :param secret: Subscription secret
        :param timeout: Request timeout in seconds
        :return: Subscription response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/guards/{secret}/info", timeout=timeout
        )
        data = await response.json()
        return GuardSubscriptionResponse(**data)

    @classmethod
    async def get_subscription_usages_by_secret(
        cls, host: str, secret: str, timeout: int = 10
    ) -> GuardSubscriptionUsageLogsResponse:
        """
        Get subscription usage logs by secret.

        :param host: API host URL
        :param secret: Subscription secret
        :param timeout: Request timeout in seconds
        :return: Subscription usage logs response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/guards/{secret}/usages", timeout=timeout
        )
        data = await response.json()
        return GuardSubscriptionUsageLogsResponse(**data)

    @classmethod
    async def get_stats(
        cls, host: str, token: str, timeout: int = 10
    ) -> GuardStatsResponse:
        """
        Get general statistics.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Stats response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/stats",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardStatsResponse(**data)

    @classmethod
    async def get_admin_stats(
        cls, host: str, token: str, timeout: int = 10
    ) -> AdminStatsResponseNew:
        """
        Get all admin statistics in one endpoint.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Admin stats response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/stats/admin",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return AdminStatsResponseNew(**data)

    @classmethod
    async def get_nodes(
        cls, host: str, token: str, timeout: int = 10
    ) -> List[GuardNodeResponse]:
        """
        Get a list of all nodes.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: List of node responses
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/nodes",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [GuardNodeResponse(**item) for item in data]

    @classmethod
    async def create_node(
        cls, host: str, token: str, node_data: GuardNodeCreate, timeout: int = 10
    ) -> GuardNodeResponse:
        """
        Create a new node.

        :param host: API host URL
        :param token: Authentication token
        :param node_data: Node creation data
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/nodes",
            data=node_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardNodeResponse(**data)

    @classmethod
    async def get_node_stats(
        cls, host: str, token: str, timeout: int = 10
    ) -> GuardNodeStatsResponse:
        """
        Get statistics about nodes.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Node stats response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/nodes/stats",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardNodeStatsResponse(**data)

    @classmethod
    async def get_node(
        cls, host: str, token: str, node_id: int, timeout: int = 10
    ) -> GuardNodeResponse:
        """
        Get a single node by ID.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/nodes/{node_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardNodeResponse(**data)

    @classmethod
    async def update_node(
        cls,
        host: str,
        token: str,
        node_id: int,
        node_data: GuardNodeUpdate,
        timeout: int = 10,
    ) -> GuardNodeResponse:
        """
        Update an existing node.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param node_data: Node update data
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/nodes/{node_id}",
            data=node_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardNodeResponse(**data)

    @classmethod
    async def delete_node(
        cls, host: str, token: str, node_id: int, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Delete a node by ID.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/nodes/{node_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def enable_node(
        cls, host: str, token: str, node_id: int, timeout: int = 10
    ) -> GuardNodeResponse:
        """
        Enable a node by ID.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/nodes/{node_id}/enable",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardNodeResponse(**data)

    @classmethod
    async def disable_node(
        cls, host: str, token: str, node_id: int, timeout: int = 10
    ) -> GuardNodeResponse:
        """
        Disable a node by ID.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/nodes/{node_id}/disable",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardNodeResponse(**data)

    @classmethod
    async def get_services(
        cls, host: str, token: str, timeout: int = 10
    ) -> List[GuardServiceResponse]:
        """
        Get a list of all services.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: List of service responses
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/services",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [GuardServiceResponse(**item) for item in data]

    @classmethod
    async def create_service(
        cls, host: str, token: str, service_data: GuardServiceCreate, timeout: int = 10
    ) -> GuardServiceResponse:
        """
        Create a new service.

        :param host: API host URL
        :param token: Authentication token
        :param service_data: Service creation data
        :param timeout: Request timeout in seconds
        :return: Service response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/services",
            data=service_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardServiceResponse(**data)

    @classmethod
    async def get_service(
        cls, host: str, token: str, service_id: int, timeout: int = 10
    ) -> GuardServiceResponse:
        """
        Get a single service by ID.

        :param host: API host URL
        :param token: Authentication token
        :param service_id: Service ID
        :param timeout: Request timeout in seconds
        :return: Service response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/services/{service_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardServiceResponse(**data)

    @classmethod
    async def update_service(
        cls,
        host: str,
        token: str,
        service_id: int,
        service_data: GuardServiceUpdate,
        timeout: int = 10,
    ) -> GuardServiceResponse:
        """
        Update an existing service.

        :param host: API host URL
        :param token: Authentication token
        :param service_id: Service ID
        :param service_data: Service update data
        :param timeout: Request timeout in seconds
        :return: Service response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/services/{service_id}",
            data=service_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardServiceResponse(**data)

    @classmethod
    async def delete_service(
        cls, host: str, token: str, service_id: int, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Delete a service by ID.

        :param host: API host URL
        :param token: Authentication token
        :param service_id: Service ID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/services/{service_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_admins(
        cls, host: str, token: str, timeout: int = 10
    ) -> List[GuardAdminResponse]:
        """
        Get a list of all admins.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: List of admin responses
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/admins",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [GuardAdminResponse(**item) for item in data]

    @classmethod
    async def create_admin(
        cls, host: str, token: str, admin_data: GuardAdminCreate, timeout: int = 10
    ) -> GuardAdminResponse:
        """
        Create a new admin.

        :param host: API host URL
        :param token: Authentication token
        :param admin_data: Admin creation data
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admins",
            data=admin_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardAdminResponse(**data)

    @classmethod
    async def create_token(
        cls, host: str, username: str, password: str, timeout: int = 10
    ) -> GuardAdminToken:
        """
        Create authentication token for admin.

        :param host: API host URL
        :param username: Admin username
        :param password: Admin password
        :param timeout: Request timeout in seconds
        :return: Admin token
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admins/token",
            data={"username": username, "password": password},
            headers={"Content-Type": "application/x-www-form-urlencoded"},
            timeout=timeout,
        )
        data = await response.json()
        return GuardAdminToken(**data)

    @classmethod
    async def get_current_admin(
        cls, host: str, token: str, timeout: int = 10
    ) -> GuardAdminResponse:
        """
        Get the current authenticated admin.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/admins/current",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardAdminResponse(**data)

    @classmethod
    async def update_current_admin(
        cls,
        host: str,
        token: str,
        admin_data: GuardAdminCurrentUpdate,
        timeout: int = 10,
    ) -> GuardAdminResponse:
        """
        Update the current admin.

        :param host: API host URL
        :param token: Authentication token
        :param admin_data: Admin update data
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/admins/current",
            data=admin_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardAdminResponse(**data)

    @classmethod
    async def get_current_admin_usages(
        cls, host: str, token: str, timeout: int = 10
    ) -> GuardAdminUsageLogsResponse:
        """
        Get usage logs for the current admin.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Admin usage logs response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/admins/current/usages",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardAdminUsageLogsResponse(**data)

    @classmethod
    async def get_admin(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> GuardAdminResponse:
        """
        Get a single admin by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/admins/{username}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardAdminResponse(**data)

    @classmethod
    async def update_admin(
        cls,
        host: str,
        token: str,
        username: str,
        admin_data: GuardAdminUpdate,
        timeout: int = 10,
    ) -> GuardAdminResponse:
        """
        Update an existing admin.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param admin_data: Admin update data
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/admins/{username}",
            data=admin_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardAdminResponse(**data)

    @classmethod
    async def delete_admin(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Delete an admin by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/admins/{username}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_admin_usages(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> GuardAdminUsageLogsResponse:
        """
        Get usage logs for an admin by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Admin usage logs response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/admins/{username}/usages",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardAdminUsageLogsResponse(**data)

    @classmethod
    async def enable_admin(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> GuardAdminResponse:
        """
        Enable an admin by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admins/{username}/enable",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardAdminResponse(**data)

    @classmethod
    async def disable_admin(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> GuardAdminResponse:
        """
        Disable an admin by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admins/{username}/disable",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardAdminResponse(**data)

    @classmethod
    async def revoke_admin_api_key(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> GuardAdminResponse:
        """
        Revoke API key for an admin by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admins/{username}/revoke",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardAdminResponse(**data)

    @classmethod
    async def get_admin_subscriptions(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> List[GuardSubscriptionResponse]:
        """
        Get subscriptions of an admin by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: List of subscription responses
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/admins/{username}/subscriptions",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [GuardSubscriptionResponse(**item) for item in data]

    @classmethod
    async def delete_admin_subscriptions(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Delete subscriptions of an admin by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/admins/{username}/subscriptions",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def activate_admin_subscriptions(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Activate subscriptions of an admin by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admins/{username}/subscriptions/activate",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def deactivate_admin_subscriptions(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Deactivate subscriptions of an admin by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admins/{username}/subscriptions/deactivate",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_subscriptions(
        cls,
        host: str,
        token: str,
        limited: Optional[bool] = None,
        expired: Optional[bool] = None,
        is_active: Optional[bool] = None,
        enabled: Optional[bool] = None,
        search: Optional[str] = None,
        online: Optional[bool] = None,
        order_by: Optional[str] = None,
        page: int = 1,
        size: int = 10,
        timeout: int = 10,
    ) -> List[GuardSubscriptionResponse]:
        """
        Get a list of all subscriptions with optional filters.

        :param host: API host URL
        :param token: Authentication token
        :param limited: Filter by limited status
        :param expired: Filter by expired status
        :param is_active: Filter by active status
        :param enabled: Filter by enabled status
        :param search: Search query
        :param online: Filter by online status
        :param order_by: Order by field
        :param page: Page number
        :param size: Page size
        :param timeout: Request timeout in seconds
        :return: List of subscription responses
        """
        params = {"page": page, "size": size}
        if limited is not None:
            params["limited"] = limited
        if expired is not None:
            params["expired"] = expired
        if is_active is not None:
            params["is_active"] = is_active
        if enabled is not None:
            params["enabled"] = enabled
        if search is not None:
            params["search"] = search
        if online is not None:
            params["online"] = online
        if order_by is not None:
            params["order_by"] = order_by

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/subscriptions",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [GuardSubscriptionResponse(**item) for item in data]

    @classmethod
    async def create_subscriptions(
        cls,
        host: str,
        token: str,
        subscriptions_data: List[GuardSubscriptionCreate],
        timeout: int = 10,
    ) -> List[GuardSubscriptionResponse]:
        """
        Create bulk subscriptions.

        :param host: API host URL
        :param token: Authentication token
        :param subscriptions_data: List of subscription creation data
        :param timeout: Request timeout in seconds
        :return: List of subscription responses
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/subscriptions",
            data=[sub.dict() for sub in subscriptions_data],
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        response_data = await response.json()
        return [GuardSubscriptionResponse(**item) for item in response_data]

    @classmethod
    async def get_subscription_count(
        cls,
        host: str,
        token: str,
        limited: Optional[bool] = None,
        expired: Optional[bool] = None,
        is_active: Optional[bool] = None,
        enabled: Optional[bool] = None,
        online: Optional[bool] = None,
        timeout: int = 10,
    ) -> int:
        """
        Get the count of subscriptions.

        :param host: API host URL
        :param token: Authentication token
        :param limited: Filter by limited status
        :param expired: Filter by expired status
        :param is_active: Filter by active status
        :param enabled: Filter by enabled status
        :param online: Filter by online status
        :param timeout: Request timeout in seconds
        :return: Subscription count
        """
        params = {}
        if limited is not None:
            params["limited"] = limited
        if expired is not None:
            params["expired"] = expired
        if is_active is not None:
            params["is_active"] = is_active
        if enabled is not None:
            params["enabled"] = enabled
        if online is not None:
            params["online"] = online

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/subscriptions/count",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_subscription_stats(
        cls, host: str, token: str, timeout: int = 10
    ) -> GuardSubscriptionStatsResponse:
        """
        Get subscription statistics.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Subscription stats response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/subscriptions/stats",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardSubscriptionStatsResponse(**data)

    @classmethod
    async def get_subscription(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> GuardSubscriptionResponse:
        """
        Get a single subscription by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Subscription username
        :param timeout: Request timeout in seconds
        :return: Subscription response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/subscriptions/{username}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardSubscriptionResponse(**data)

    @classmethod
    async def update_subscription(
        cls,
        host: str,
        token: str,
        username: str,
        subscription_data: GuardSubscriptionUpdate,
        timeout: int = 10,
    ) -> GuardSubscriptionResponse:
        """
        Update an existing subscription.

        :param host: API host URL
        :param token: Authentication token
        :param username: Subscription username
        :param subscription_data: Subscription update data
        :param timeout: Request timeout in seconds
        :return: Subscription response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/subscriptions/{username}",
            data=subscription_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardSubscriptionResponse(**data)

    @classmethod
    async def delete_subscription(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Delete a subscription by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Subscription username
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/subscriptions/{username}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_subscription_usages(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> GuardSubscriptionUsageLogsResponse:
        """
        Get usage logs for a subscription by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Subscription username
        :param timeout: Request timeout in seconds
        :return: Subscription usage logs response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/subscriptions/{username}/usages",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardSubscriptionUsageLogsResponse(**data)

    @classmethod
    async def enable_subscription(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> GuardSubscriptionResponse:
        """
        Enable a subscription by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Subscription username
        :param timeout: Request timeout in seconds
        :return: Subscription response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/subscriptions/{username}/enable",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardSubscriptionResponse(**data)

    @classmethod
    async def disable_subscription(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> GuardSubscriptionResponse:
        """
        Disable a subscription by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Subscription username
        :param timeout: Request timeout in seconds
        :return: Subscription response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/subscriptions/{username}/disable",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardSubscriptionResponse(**data)

    @classmethod
    async def revoke_subscription(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> GuardSubscriptionResponse:
        """
        Revoke a subscription by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Subscription username
        :param timeout: Request timeout in seconds
        :return: Subscription response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/subscriptions/{username}/revoke",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardSubscriptionResponse(**data)

    @classmethod
    async def reset_subscription(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> GuardSubscriptionResponse:
        """
        Reset a subscription by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Subscription username
        :param timeout: Request timeout in seconds
        :return: Subscription response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/subscriptions/{username}/reset",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return GuardSubscriptionResponse(**data)

    @classmethod
    async def bulk_add_service(
        cls, host: str, token: str, service_id: int, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Add a service to all subscriptions of the current admin.

        :param host: API host URL
        :param token: Authentication token
        :param service_id: Service ID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/subscriptions/services/{service_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def bulk_remove_service(
        cls, host: str, token: str, service_id: int, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Remove a service from all subscriptions of the current admin.

        :param host: API host URL
        :param token: Authentication token
        :param service_id: Service ID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/subscriptions/services/{service_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()



from .manager import GuardManager
from .types import (
    GuardAdminCreate,
    GuardAdminCurrentUpdate,
    GuardAdminPlaceHolder,
    GuardAdminPlaceHolderCategory,
    GuardAdminResponse,
    GuardAdminRole,
    GuardAdminToken,
    GuardAdminUpdate,
    GuardAdminUsageLog,
    GuardAdminUsageLogsResponse,
    GuardNodeCreate,
    GuardNodeResponse,
    GuardNodeStatsResponse,
    GuardNodeUpdate,
    GuardNodeCategory,
    GuardServiceCreate,
    GuardServiceResponse,
    GuardServiceUpdate,
    AdminStatsResponseNew,
    GuardCountDetailStats,
    GuardExpireSubDetailStats,
    GuardStatsResponse,
    GuardTopSubDetailStats,
    GuardUsageDetailStats,
    GuardSubscriptionCreate,
    GuardSubscriptionResponse,
    GuardSubscriptionStatsResponse,
    GuardSubscriptionUpdate,
    GuardSubscriptionUsageLog,
    GuardSubscriptionUsageLogsResponse,
)

__all__ = [
    "GuardManager",
    "AdminStatsResponseNew",
    "GuardAdminCreate",
    "GuardAdminCurrentUpdate",
    "GuardAdminPlaceHolder",
    "GuardAdminPlaceHolderCategory",
    "GuardAdminResponse",
    "GuardAdminRole",
    "GuardAdminToken",
    "GuardAdminUpdate",
    "GuardAdminUsageLog",
    "GuardAdminUsageLogsResponse",
    "GuardNodeCreate",
    "GuardNodeResponse",
    "GuardNodeStatsResponse",
    "GuardNodeUpdate",
    "GuardNodeCategory",
    "GuardServiceCreate",
    "GuardServiceResponse",
    "GuardServiceUpdate",
    "GuardCountDetailStats",
    "GuardExpireSubDetailStats",
    "GuardStatsResponse",
    "GuardTopSubDetailStats",
    "GuardUsageDetailStats",
    "GuardSubscriptionCreate",
    "GuardSubscriptionResponse",
    "GuardSubscriptionStatsResponse",
    "GuardSubscriptionUpdate",
    "GuardSubscriptionUsageLog",
    "GuardSubscriptionUsageLogsResponse",
]



from typing import List, Optional
from pydantic import BaseModel, Field


class GuardServiceCreate(BaseModel):
    """Schema for creating a new service"""

    remark: str = Field(..., title="Remark")
    node_ids: List[int] = Field(..., title="Node Ids")


class GuardServiceUpdate(BaseModel):
    """Schema for updating an existing service"""

    remark: Optional[str] = Field(None, title="Remark")
    node_ids: Optional[List[int]] = Field(None, title="Node Ids")


class GuardServiceResponse(BaseModel):
    """Service response schema"""

    id: int = Field(..., title="Id")
    remark: str = Field(..., title="Remark")
    node_ids: List[int] = Field(..., title="Node Ids")



from pydantic import BaseModel, Field


class RemnawaveSubscription(BaseModel):
    """Subscription response"""

    short_uuid: str = Field(..., alias="shortUuid")
    username: str
    days_left: int = Field(..., alias="daysLeft")
    traffic_used: str = Field(..., alias="trafficUsed")
    traffic_limit: str = Field(..., alias="trafficLimit")
    lifetime_traffic_used: str = Field(..., alias="lifetimeTrafficUsed")
    expires_at: str = Field(..., alias="expiresAt")
    is_active: bool = Field(..., alias="isActive")
    user_status: str = Field(..., alias="userStatus")
    subscription_url: str = Field(..., alias="subscriptionUrl")

    class Config:
        populate_by_name = True



from typing import Optional
from pydantic import BaseModel, Field
from enum import Enum


class PasarGuardNodeStatus(str, Enum):
    """Node status enumeration"""

    connected = "connected"
    connecting = "connecting"
    error = "error"
    disabled = "disabled"
    limited = "limited"


class PasarGuardNodeConnectionType(str, Enum):
    """Node connection type"""

    grpc = "grpc"
    rest = "rest"


class PasarGuardDataLimitResetStrategy(str, Enum):
    """Data limit reset strategy"""

    no_reset = "no_reset"
    day = "day"
    week = "week"
    month = "month"
    year = "year"


class PasarGuardNodeCreate(BaseModel):
    """Schema for creating a new node"""

    name: str = Field(..., title="Name")
    address: str = Field(..., title="Address")
    port: int = Field(62050, title="Port")
    usage_coefficient: float = Field(1.0, title="Usage Coefficient")
    connection_type: PasarGuardNodeConnectionType = Field(..., title="Connection Type")
    server_ca: str = Field(..., title="Server Ca")
    keep_alive: int = Field(..., title="Keep Alive")
    core_config_id: int = Field(..., title="Core Config Id")
    api_key: str = Field(..., title="Api Key")
    data_limit: int = Field(0, title="Data Limit")
    data_limit_reset_strategy: PasarGuardDataLimitResetStrategy = Field(
        PasarGuardDataLimitResetStrategy.no_reset, title="Data Limit Reset Strategy"
    )
    reset_time: int = Field(-1, title="Reset Time")


class PasarGuardNodeModify(BaseModel):
    """Schema for modifying an existing node"""

    name: Optional[str] = Field(None, title="Name")
    address: Optional[str] = Field(None, title="Address")
    port: Optional[int] = Field(None, title="Port")
    usage_coefficient: Optional[float] = Field(None, title="Usage Coefficient")
    connection_type: Optional[PasarGuardNodeConnectionType] = Field(
        None, title="Connection Type"
    )
    server_ca: Optional[str] = Field(None, title="Server Ca")
    keep_alive: Optional[int] = Field(None, title="Keep Alive")
    core_config_id: Optional[int] = Field(None, title="Core Config Id")
    api_key: Optional[str] = Field(None, title="Api Key")
    data_limit: Optional[int] = Field(None, title="Data Limit")
    data_limit_reset_strategy: Optional[PasarGuardDataLimitResetStrategy] = Field(
        None, title="Data Limit Reset Strategy"
    )
    reset_time: Optional[int] = Field(None, title="Reset Time")
    status: Optional[PasarGuardNodeStatus] = Field(None, title="Status")


class PasarGuardNodeResponse(BaseModel):
    """Node response schema"""

    id: int = Field(..., title="Id")
    name: str = Field(..., title="Name")
    address: str = Field(..., title="Address")
    port: int = Field(62050, title="Port")
    usage_coefficient: float = Field(1.0, title="Usage Coefficient")
    connection_type: PasarGuardNodeConnectionType = Field(..., title="Connection Type")
    server_ca: str = Field(..., title="Server Ca")
    keep_alive: int = Field(..., title="Keep Alive")
    core_config_id: Optional[int] = Field(None, title="Core Config Id")
    api_key: Optional[str] = Field(None, title="Api Key")
    data_limit: int = Field(0, title="Data Limit")
    data_limit_reset_strategy: PasarGuardDataLimitResetStrategy = Field(
        PasarGuardDataLimitResetStrategy.no_reset, title="Data Limit Reset Strategy"
    )
    reset_time: int = Field(-1, title="Reset Time")
    xray_version: Optional[str] = Field(None, title="Xray Version")
    node_version: Optional[str] = Field(None, title="Node Version")
    status: PasarGuardNodeStatus = Field(..., title="Status")
    message: Optional[str] = Field(None, title="Message")
    uplink: int = Field(0, title="Uplink")
    downlink: int = Field(0, title="Downlink")
    lifetime_uplink: Optional[int] = Field(None, title="Lifetime Uplink")
    lifetime_downlink: Optional[int] = Field(None, title="Lifetime Downlink")


class PasarGuardNodeSettings(BaseModel):
    """Node settings response schema"""

    min_node_version: str = Field("v1.0.0", title="Min Node Version")



from typing import Optional, Dict, List
from datetime import datetime
from pydantic import BaseModel, Field
from .user import (
    MarzbanProxyTypes,
    MarzbanUserStatus,
    MarzbanUserDataLimitResetStrategy,
    MarzbanNextPlanModel,
)


class MarzbanSubscriptionUserResponse(BaseModel):
    """Subscription user response schema"""

    username: str = Field(..., title="Username")
    status: MarzbanUserStatus = Field(..., title="Status")
    proxies: Dict[MarzbanProxyTypes, Dict[str, str]] = Field(
        default_factory=dict, title="Proxies"
    )
    expire: Optional[int] = Field(None, title="Expire")
    data_limit: Optional[int] = Field(None, title="Data Limit")
    data_limit_reset_strategy: MarzbanUserDataLimitResetStrategy = Field(
        MarzbanUserDataLimitResetStrategy.NO_RESET, title="Data Limit Reset Strategy"
    )
    sub_updated_at: Optional[datetime] = Field(None, title="Sub Updated At")
    sub_last_user_agent: Optional[str] = Field(None, title="Sub Last User Agent")
    online_at: Optional[datetime] = Field(None, title="Online At")
    on_hold_expire_duration: Optional[int] = Field(
        None, title="On Hold Expire Duration"
    )
    on_hold_timeout: Optional[datetime] = Field(None, title="On Hold Timeout")
    next_plan: Optional[MarzbanNextPlanModel] = Field(None, title="Next Plan")
    used_traffic: int = Field(..., title="Used Traffic")
    lifetime_used_traffic: int = Field(0, title="Lifetime Used Traffic")
    created_at: datetime = Field(..., title="Created At")
    links: List[str] = Field(default_factory=list, title="Links")
    subscription_url: str = Field("", title="Subscription Url")



from typing import Optional, List, Dict, Any
from opexcore.core import RequestBase
from .types import (
    MarzneshinAdminCreate,
    MarzneshinAdminPartialModify,
    MarzneshinAdmin,
    MarzneshinAdminResponse,
    MarzneshinToken,
    MarzneshinNodeCreate,
    MarzneshinNodeModify,
    MarzneshinNodeResponse,
    MarzneshinNodeSettings,
    MarzneshinServiceCreate,
    MarzneshinServiceModify,
    MarzneshinServiceResponse,
    MarzneshinUserCreate,
    MarzneshinUserModify,
    MarzneshinUserResponse,
    MarzneshinUsersSortingOptions,
    MarzneshinUserUsageSeriesResponse,
    MarzneshinInboundHost,
    MarzneshinInboundHostResponse,
    MarzneshinInbound,
    MarzneshinBackendConfig,
    MarzneshinBackendStats,
    MarzneshinSubscriptionSettings,
    MarzneshinTelegramSettings,
    MarzneshinAdminsStats,
    MarzneshinNodesStats,
    MarzneshinUsersStats,
    MarzneshinTrafficUsageSeries,
)


class MarzneshinManager(RequestBase):
    """Marzneshin API Manager with all endpoints"""

    @classmethod
    def _generate_headers(cls, token: Optional[str] = None) -> Dict[str, str]:
        """Generate headers for API requests"""
        headers = {"Content-Type": "application/json"}
        if token:
            headers["Authorization"] = f"Bearer {token}"
        return headers

    @classmethod
    async def admin_token(
        cls, host: str, username: str, password: str, timeout: int = 10
    ) -> MarzneshinToken:
        """
        Authenticate an admin and issue a token.

        :param host: API host URL
        :param username: Admin username
        :param password: Admin password
        :param timeout: Request timeout in seconds
        :return: Token response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admins/token",
            data={"username": username, "password": password},
            headers={"Content-Type": "application/x-www-form-urlencoded"},
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinToken(**data)

    @classmethod
    async def get_current_admin(
        cls, host: str, token: str, timeout: int = 10
    ) -> MarzneshinAdmin:
        """
        Retrieve the current authenticated admin.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/admins/current",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinAdmin(**data)

    @classmethod
    async def get_admins(
        cls,
        host: str,
        token: str,
        username: Optional[str] = None,
        page: int = 1,
        size: int = 50,
        timeout: int = 10,
    ) -> list[MarzneshinAdminResponse]:
        """
        Fetch a list of admins with optional filters.

        :param host: API host URL
        :param token: Authentication token
        :param username: Filter by username
        :param page: Page number
        :param size: Page size
        :param timeout: Request timeout in seconds
        :return: List of admin responses
        """
        params = {"page": page, "size": size}
        if username:
            params["username"] = username

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/admins",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [MarzneshinAdminResponse(**admin) for admin in data.get("items", [])]

    @classmethod
    async def create_admin(
        cls, host: str, token: str, admin_data: MarzneshinAdminCreate, timeout: int = 10
    ) -> MarzneshinAdmin:
        """
        Create a new admin.

        :param host: API host URL
        :param token: Authentication token
        :param admin_data: Admin creation data
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admins",
            data=admin_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinAdmin(**data)

    @classmethod
    async def get_admin(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> MarzneshinAdminResponse:
        """
        Get admin by username.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/admins/{username}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinAdminResponse(**data)

    @classmethod
    async def modify_admin(
        cls,
        host: str,
        token: str,
        username: str,
        admin_data: MarzneshinAdminPartialModify,
        timeout: int = 10,
    ) -> MarzneshinAdminResponse:
        """
        Modify an existing admin's details.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param admin_data: Admin modification data
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/admins/{username}",
            data=admin_data.dict(exclude_none=True),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinAdminResponse(**data)

    @classmethod
    async def remove_admin(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Remove an admin from the database.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/admins/{username}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_admin_services(
        cls,
        host: str,
        token: str,
        username: str,
        page: int = 1,
        size: int = 50,
        timeout: int = 10,
    ) -> list[MarzneshinServiceResponse]:
        """
        Get admin services.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param page: Page number
        :param size: Page size
        :param timeout: Request timeout in seconds
        :return: List of service responses
        """
        params = {"page": page, "size": size}
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/admins/{username}/services",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [
            MarzneshinServiceResponse(**service) for service in data.get("items", [])
        ]

    @classmethod
    async def get_admin_users(
        cls,
        host: str,
        token: str,
        username: str,
        page: int = 1,
        size: int = 50,
        timeout: int = 10,
    ) -> list[MarzneshinUserResponse]:
        """
        Get admin users.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param page: Page number
        :param size: Page size
        :param timeout: Request timeout in seconds
        :return: List of user responses
        """
        params = {"page": page, "size": size}
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/admins/{username}/users",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [MarzneshinUserResponse(**user) for user in data.get("items", [])]

    @classmethod
    async def disable_admin_users(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> MarzneshinAdminResponse:
        """
        Disable all users of an admin.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admins/{username}/disable_users",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinAdminResponse(**data)

    @classmethod
    async def enable_admin_users(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> MarzneshinAdminResponse:
        """
        Enable all users of an admin.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admins/{username}/enable_users",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinAdminResponse(**data)

    @classmethod
    async def get_nodes(
        cls,
        host: str,
        token: str,
        status: Optional[List[str]] = None,
        name: Optional[str] = None,
        page: int = 1,
        size: int = 50,
        timeout: int = 10,
    ) -> list[MarzneshinNodeResponse]:
        """
        Retrieve a list of all nodes.

        :param host: API host URL
        :param token: Authentication token
        :param status: Filter by status
        :param name: Filter by name
        :param page: Page number
        :param size: Page size
        :param timeout: Request timeout in seconds
        :return: List of node responses
        """
        params = {"page": page, "size": size}
        if status:
            params["status"] = status
        if name:
            params["name"] = name

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/nodes",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [MarzneshinNodeResponse(**node) for node in data.get("items", [])]

    @classmethod
    async def add_node(
        cls, host: str, token: str, node_data: MarzneshinNodeCreate, timeout: int = 10
    ) -> MarzneshinNodeResponse:
        """
        Add a new node to the database.

        :param host: API host URL
        :param token: Authentication token
        :param node_data: Node creation data
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/nodes",
            data=node_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinNodeResponse(**data)

    @classmethod
    async def get_node_settings(
        cls, host: str, token: str, timeout: int = 10
    ) -> MarzneshinNodeSettings:
        """
        Retrieve the current node settings.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Node settings response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/nodes/settings",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinNodeSettings(**data)

    @classmethod
    async def get_node(
        cls, host: str, token: str, node_id: int, timeout: int = 10
    ) -> MarzneshinNodeResponse:
        """
        Retrieve details of a specific node by its ID.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/nodes/{node_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinNodeResponse(**data)

    @classmethod
    async def modify_node(
        cls,
        host: str,
        token: str,
        node_id: int,
        node_data: MarzneshinNodeModify,
        timeout: int = 10,
    ) -> MarzneshinNodeResponse:
        """
        Update a node's details.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param node_data: Node modification data
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/nodes/{node_id}",
            data=node_data.dict(exclude_none=True),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinNodeResponse(**data)

    @classmethod
    async def remove_node(
        cls, host: str, token: str, node_id: int, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Delete a node.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/nodes/{node_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def reconnect_node(
        cls, host: str, token: str, node_id: int, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Trigger a reconnection for the specified node.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/nodes/{node_id}/resync",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_node_usage(
        cls,
        host: str,
        token: str,
        node_id: int,
        start: Optional[str] = None,
        end: Optional[str] = None,
        timeout: int = 10,
    ) -> MarzneshinTrafficUsageSeries:
        """
        Get node usage statistics.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param start: Start date
        :param end: End date
        :param timeout: Request timeout in seconds
        :return: Traffic usage series
        """
        params = {}
        if start:
            params["start"] = start
        if end:
            params["end"] = end

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/nodes/{node_id}/usage",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinTrafficUsageSeries(**data)

    @classmethod
    async def get_backend_stats(
        cls, host: str, token: str, node_id: int, backend: str, timeout: int = 10
    ) -> MarzneshinBackendStats:
        """
        Get backend stats for a node.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param backend: Backend name
        :param timeout: Request timeout in seconds
        :return: Backend stats
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/nodes/{node_id}/{backend}/stats",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinBackendStats(**data)

    @classmethod
    async def get_node_backend_config(
        cls, host: str, token: str, node_id: int, backend: str, timeout: int = 10
    ) -> MarzneshinBackendConfig:
        """
        Get node backend configuration.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param backend: Backend name
        :param timeout: Request timeout in seconds
        :return: Backend config
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/nodes/{node_id}/{backend}/config",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinBackendConfig(**data)

    @classmethod
    async def alter_node_backend_config(
        cls,
        host: str,
        token: str,
        node_id: int,
        backend: str,
        config_data: MarzneshinBackendConfig,
        timeout: int = 10,
    ) -> Dict[str, Any]:
        """
        Alter node backend configuration.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param backend: Backend name
        :param config_data: Backend config data
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/nodes/{node_id}/{backend}/config",
            data=config_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_services(
        cls,
        host: str,
        token: str,
        name: Optional[str] = None,
        page: int = 1,
        size: int = 50,
        timeout: int = 10,
    ) -> list[MarzneshinServiceResponse]:
        """
        Get all services with optional filters.

        :param host: API host URL
        :param token: Authentication token
        :param name: Filter by name
        :param page: Page number
        :param size: Page size
        :param timeout: Request timeout in seconds
        :return: List of service responses
        """
        params = {"page": page, "size": size}
        if name:
            params["name"] = name

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/services",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [
            MarzneshinServiceResponse(**service) for service in data.get("items", [])
        ]

    @classmethod
    async def add_service(
        cls,
        host: str,
        token: str,
        service_data: MarzneshinServiceCreate,
        timeout: int = 10,
    ) -> MarzneshinServiceResponse:
        """
        Add a new service.

        :param host: API host URL
        :param token: Authentication token
        :param service_data: Service creation data
        :param timeout: Request timeout in seconds
        :return: Service response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/services",
            data=service_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinServiceResponse(**data)

    @classmethod
    async def get_service(
        cls, host: str, token: str, service_id: int, timeout: int = 10
    ) -> MarzneshinServiceResponse:
        """
        Get service information by ID.

        :param host: API host URL
        :param token: Authentication token
        :param service_id: Service ID
        :param timeout: Request timeout in seconds
        :return: Service response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/services/{service_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinServiceResponse(**data)

    @classmethod
    async def modify_service(
        cls,
        host: str,
        token: str,
        service_id: int,
        service_data: MarzneshinServiceModify,
        timeout: int = 10,
    ) -> MarzneshinServiceResponse:
        """
        Modify a service.

        :param host: API host URL
        :param token: Authentication token
        :param service_id: Service ID
        :param service_data: Service modification data
        :param timeout: Request timeout in seconds
        :return: Service response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/services/{service_id}",
            data=service_data.dict(exclude_none=True),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinServiceResponse(**data)

    @classmethod
    async def remove_service(
        cls, host: str, token: str, service_id: int, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Remove a service.

        :param host: API host URL
        :param token: Authentication token
        :param service_id: Service ID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/services/{service_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_service_users(
        cls,
        host: str,
        token: str,
        service_id: int,
        page: int = 1,
        size: int = 50,
        timeout: int = 10,
    ) -> list[MarzneshinUserResponse]:
        """
        Get service users.

        :param host: API host URL
        :param token: Authentication token
        :param service_id: Service ID
        :param page: Page number
        :param size: Page size
        :param timeout: Request timeout in seconds
        :return: List of user responses
        """
        params = {"page": page, "size": size}
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/services/{service_id}/users",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [MarzneshinUserResponse(**user) for user in data.get("items", [])]

    @classmethod
    async def get_service_inbounds(
        cls,
        host: str,
        token: str,
        service_id: int,
        page: int = 1,
        size: int = 50,
        timeout: int = 10,
    ) -> list[MarzneshinInbound]:
        """
        Get service inbounds.

        :param host: API host URL
        :param token: Authentication token
        :param service_id: Service ID
        :param page: Page number
        :param size: Page size
        :param timeout: Request timeout in seconds
        :return: List of inbound responses
        """
        params = {"page": page, "size": size}
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/services/{service_id}/inbounds",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [MarzneshinInbound(**inbound) for inbound in data.get("items", [])]

    @classmethod
    async def get_inbounds(
        cls,
        host: str,
        token: str,
        tag: Optional[str] = None,
        page: int = 1,
        size: int = 50,
        timeout: int = 10,
    ) -> list[MarzneshinInbound]:
        """
        Get all inbounds.

        :param host: API host URL
        :param token: Authentication token
        :param tag: Filter by tag
        :param page: Page number
        :param size: Page size
        :param timeout: Request timeout in seconds
        :return: List of inbound responses
        """
        params = {"page": page, "size": size}
        if tag:
            params["tag"] = tag

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/inbounds",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [MarzneshinInbound(**inbound) for inbound in data.get("items", [])]

    @classmethod
    async def get_inbound(
        cls, host: str, token: str, inbound_id: int, timeout: int = 10
    ) -> MarzneshinInbound:
        """
        Get a specific inbound.

        :param host: API host URL
        :param token: Authentication token
        :param inbound_id: Inbound ID
        :param timeout: Request timeout in seconds
        :return: Inbound response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/inbounds/{inbound_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinInbound(**data)

    @classmethod
    async def get_hosts(
        cls, host: str, token: str, page: int = 1, size: int = 50, timeout: int = 10
    ) -> list[MarzneshinInboundHostResponse]:
        """
        Get all hosts.

        :param host: API host URL
        :param token: Authentication token
        :param page: Page number
        :param size: Page size
        :param timeout: Request timeout in seconds
        :return: List of host responses
        """
        params = {"page": page, "size": size}
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/inbounds/hosts",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [MarzneshinInboundHostResponse(**host) for host in data.get("items", [])]

    @classmethod
    async def create_unbound_host(
        cls,
        host: str,
        token: str,
        host_data: MarzneshinInboundHost,
        timeout: int = 10,
    ) -> MarzneshinInboundHostResponse:
        """
        Create a host without an inbound.

        :param host: API host URL
        :param token: Authentication token
        :param host_data: Host creation data
        :param timeout: Request timeout in seconds
        :return: Host response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/inbounds/hosts",
            data=host_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinInboundHostResponse(**data)

    @classmethod
    async def get_host(
        cls, host: str, token: str, host_id: int, timeout: int = 10
    ) -> MarzneshinInboundHostResponse:
        """
        Get a host.

        :param host: API host URL
        :param token: Authentication token
        :param host_id: Host ID
        :param timeout: Request timeout in seconds
        :return: Host response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/inbounds/hosts/{host_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinInboundHostResponse(**data)

    @classmethod
    async def update_host(
        cls,
        host: str,
        token: str,
        host_id: int,
        host_data: MarzneshinInboundHost,
        timeout: int = 10,
    ) -> MarzneshinInboundHostResponse:
        """
        Update a host.

        :param host: API host URL
        :param token: Authentication token
        :param host_id: Host ID
        :param host_data: Host update data
        :param timeout: Request timeout in seconds
        :return: Host response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/inbounds/hosts/{host_id}",
            data=host_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinInboundHostResponse(**data)

    @classmethod
    async def delete_host(
        cls, host: str, token: str, host_id: int, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Delete a host.

        :param host: API host URL
        :param token: Authentication token
        :param host_id: Host ID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/inbounds/hosts/{host_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_inbound_hosts(
        cls,
        host: str,
        token: str,
        inbound_id: int,
        page: int = 1,
        size: int = 50,
        timeout: int = 10,
    ) -> list[MarzneshinInboundHostResponse]:
        """
        Get hosts of a specific inbound.

        :param host: API host URL
        :param token: Authentication token
        :param inbound_id: Inbound ID
        :param page: Page number
        :param size: Page size
        :param timeout: Request timeout in seconds
        :return: List of host responses
        """
        params = {"page": page, "size": size}
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/inbounds/{inbound_id}/hosts",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [MarzneshinInboundHostResponse(**host) for host in data.get("items", [])]

    @classmethod
    async def create_inbound_host(
        cls,
        host: str,
        token: str,
        inbound_id: int,
        host_data: MarzneshinInboundHost,
        timeout: int = 10,
    ) -> MarzneshinInboundHostResponse:
        """
        Add a host to an inbound.

        :param host: API host URL
        :param token: Authentication token
        :param inbound_id: Inbound ID
        :param host_data: Host creation data
        :param timeout: Request timeout in seconds
        :return: Host response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/inbounds/{inbound_id}/hosts",
            data=host_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinInboundHostResponse(**data)

    @classmethod
    async def get_users(
        cls,
        host: str,
        token: str,
        username: Optional[List[str]] = None,
        order_by: Optional[MarzneshinUsersSortingOptions] = None,
        descending: bool = False,
        is_active: Optional[bool] = None,
        activated: Optional[bool] = None,
        expired: Optional[bool] = None,
        data_limit_reached: Optional[bool] = None,
        enabled: Optional[bool] = None,
        owner_username: Optional[str] = None,
        page: int = 1,
        size: int = 50,
        timeout: int = 10,
    ) -> list[MarzneshinUserResponse]:
        """
        Get all users with optional filters.

        :param host: API host URL
        :param token: Authentication token
        :param username: Filter by usernames
        :param order_by: Sort field
        :param descending: Sort in descending order
        :param is_active: Filter by active status
        :param activated: Filter by activated status
        :param expired: Filter by expired status
        :param data_limit_reached: Filter by data limit reached status
        :param enabled: Filter by enabled status
        :param owner_username: Filter by owner username
        :param page: Page number
        :param size: Page size
        :param timeout: Request timeout in seconds
        :return: List of user responses
        """
        params = {"page": page, "size": size, "descending": descending}
        if username:
            params["username"] = username
        if order_by:
            params["order_by"] = order_by.value
        if is_active is not None:
            params["is_active"] = is_active
        if activated is not None:
            params["activated"] = activated
        if expired is not None:
            params["expired"] = expired
        if data_limit_reached is not None:
            params["data_limit_reached"] = data_limit_reached
        if enabled is not None:
            params["enabled"] = enabled
        if owner_username:
            params["owner_username"] = owner_username

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/users",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [MarzneshinUserResponse(**user) for user in data.get("items", [])]

    @classmethod
    async def add_user(
        cls, host: str, token: str, user_data: MarzneshinUserCreate, timeout: int = 10
    ) -> MarzneshinUserResponse:
        """
        Add a new user.

        :param host: API host URL
        :param token: Authentication token
        :param user_data: User creation data
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/users",
            data=user_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinUserResponse(**data)

    @classmethod
    async def reset_users_data_usage(
        cls, host: str, token: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Reset all users data usage.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/users/reset",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def delete_expired_users(
        cls, host: str, token: str, passed_time: int, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Delete expired users.

        :param host: API host URL
        :param token: Authentication token
        :param passed_time: Number of days passed since expiration
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        params = {"passed_time": passed_time}
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/users/expired",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_user(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> MarzneshinUserResponse:
        """
        Get user information.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/users/{username}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinUserResponse(**data)

    @classmethod
    async def modify_user(
        cls,
        host: str,
        token: str,
        username: str,
        user_data: MarzneshinUserModify,
        timeout: int = 10,
    ) -> MarzneshinUserResponse:
        """
        Modify an existing user.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param user_data: User modification data
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/users/{username}",
            data=user_data.dict(exclude_none=True),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinUserResponse(**data)

    @classmethod
    async def remove_user(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Remove a user.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/users/{username}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_user_services(
        cls,
        host: str,
        token: str,
        username: str,
        page: int = 1,
        size: int = 50,
        timeout: int = 10,
    ) -> list[MarzneshinServiceResponse]:
        """
        Get user services.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param page: Page number
        :param size: Page size
        :param timeout: Request timeout in seconds
        :return: List of service responses
        """
        params = {"page": page, "size": size}
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/users/{username}/services",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [
            MarzneshinServiceResponse(**service) for service in data.get("items", [])
        ]

    @classmethod
    async def reset_user_data_usage(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> MarzneshinUserResponse:
        """
        Reset user data usage.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/users/{username}/reset",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinUserResponse(**data)

    @classmethod
    async def enable_user(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> MarzneshinUserResponse:
        """
        Enable a user.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/users/{username}/enable",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinUserResponse(**data)

    @classmethod
    async def disable_user(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> MarzneshinUserResponse:
        """
        Disable a user.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/users/{username}/disable",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinUserResponse(**data)

    @classmethod
    async def revoke_user_subscription(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> MarzneshinUserResponse:
        """
        Revoke user subscription.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/users/{username}/revoke_sub",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinUserResponse(**data)

    @classmethod
    async def get_user_usage(
        cls,
        host: str,
        token: str,
        username: str,
        start: Optional[str] = None,
        end: Optional[str] = None,
        timeout: int = 10,
    ) -> MarzneshinUserUsageSeriesResponse:
        """
        Get user usage statistics.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param start: Start date
        :param end: End date
        :param timeout: Request timeout in seconds
        :return: User usage series response
        """
        params = {}
        if start:
            params["start"] = start
        if end:
            params["end"] = end

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/users/{username}/usage",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinUserUsageSeriesResponse(**data)

    @classmethod
    async def set_user_owner(
        cls,
        host: str,
        token: str,
        username: str,
        admin_username: str,
        timeout: int = 10,
    ) -> MarzneshinUserResponse:
        """
        Set user owner.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param admin_username: Admin username
        :param timeout: Request timeout in seconds
        :return: User response
        """
        params = {"admin_username": admin_username}
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/users/{username}/set-owner",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinUserResponse(**data)

    @classmethod
    async def user_subscription(
        cls, host: str, username: str, key: str, user_agent: str = "", timeout: int = 10
    ) -> List[str]:
        """
        Get user subscription.

        :param host: API host URL
        :param username: Username
        :param key: Subscription key
        :param user_agent: User agent header
        :param timeout: Request timeout in seconds
        :return: Subscription links
        """
        headers = {}
        if user_agent:
            headers["user-agent"] = user_agent

        response = await cls.get(
            url=f"{host.rstrip('/')}/sub/{username}/{key}",
            headers=headers,
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def user_subscription_info(
        cls, host: str, username: str, key: str, timeout: int = 10
    ) -> MarzneshinUserResponse:
        """
        Get user subscription info.

        :param host: API host URL
        :param username: Username
        :param key: Subscription key
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/sub/{username}/{key}/info",
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinUserResponse(**data)

    @classmethod
    async def user_subscription_usage(
        cls,
        host: str,
        username: str,
        key: str,
        start: Optional[str] = None,
        end: Optional[str] = None,
        timeout: int = 10,
    ) -> MarzneshinTrafficUsageSeries:
        """
        Get user subscription usage.

        :param host: API host URL
        :param username: Username
        :param key: Subscription key
        :param start: Start date
        :param end: End date
        :param timeout: Request timeout in seconds
        :return: Traffic usage series
        """
        params = {}
        if start:
            params["start"] = start
        if end:
            params["end"] = end

        response = await cls.get(
            url=f"{host.rstrip('/')}/sub/{username}/{key}/usage",
            params=params,
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinTrafficUsageSeries(**data)

    @classmethod
    async def user_subscription_with_client_type(
        cls,
        host: str,
        username: str,
        key: str,
        client_type: str,
        timeout: int = 10,
    ) -> List[str]:
        """
        Get user subscription with specific client type.

        :param host: API host URL
        :param username: Username
        :param key: Subscription key
        :param client_type: Client type (sing-box, clash-meta, clash, xray, v2ray, links, wireguard)
        :param timeout: Request timeout in seconds
        :return: Subscription links
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/sub/{username}/{key}/{client_type}",
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_subscription_settings(
        cls, host: str, token: str, timeout: int = 10
    ) -> MarzneshinSubscriptionSettings:
        """
        Get subscription settings.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Subscription settings
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/system/settings/subscription",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinSubscriptionSettings(**data)

    @classmethod
    async def update_subscription_settings(
        cls,
        host: str,
        token: str,
        settings_data: MarzneshinSubscriptionSettings,
        timeout: int = 10,
    ) -> MarzneshinSubscriptionSettings:
        """
        Update subscription settings.

        :param host: API host URL
        :param token: Authentication token
        :param settings_data: Subscription settings data
        :param timeout: Request timeout in seconds
        :return: Updated subscription settings
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/system/settings/subscription",
            data=settings_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinSubscriptionSettings(**data)

    @classmethod
    async def get_telegram_settings(
        cls, host: str, token: str, timeout: int = 10
    ) -> Optional[MarzneshinTelegramSettings]:
        """
        Get Telegram settings.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Telegram settings or None
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/system/settings/telegram",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinTelegramSettings(**data) if data else None

    @classmethod
    async def update_telegram_settings(
        cls,
        host: str,
        token: str,
        settings_data: Optional[MarzneshinTelegramSettings],
        timeout: int = 10,
    ) -> Optional[MarzneshinTelegramSettings]:
        """
        Update Telegram settings.

        :param host: API host URL
        :param token: Authentication token
        :param settings_data: Telegram settings data or None to disable
        :param timeout: Request timeout in seconds
        :return: Updated Telegram settings or None
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/system/settings/telegram",
            data=settings_data.dict() if settings_data else None,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinTelegramSettings(**data) if data else None

    @classmethod
    async def get_admins_stats(
        cls, host: str, token: str, timeout: int = 10
    ) -> MarzneshinAdminsStats:
        """
        Get admins statistics.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Admins stats
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/system/stats/admins",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinAdminsStats(**data)

    @classmethod
    async def get_nodes_stats(
        cls, host: str, token: str, timeout: int = 10
    ) -> MarzneshinNodesStats:
        """
        Get nodes statistics.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Nodes stats
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/system/stats/nodes",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinNodesStats(**data)

    @classmethod
    async def get_total_traffic_stats(
        cls,
        host: str,
        token: str,
        start: Optional[str] = None,
        end: Optional[str] = None,
        timeout: int = 10,
    ) -> MarzneshinTrafficUsageSeries:
        """
        Get total traffic statistics.

        :param host: API host URL
        :param token: Authentication token
        :param start: Start date
        :param end: End date
        :param timeout: Request timeout in seconds
        :return: Traffic usage series
        """
        params = {}
        if start:
            params["start"] = start
        if end:
            params["end"] = end

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/system/stats/traffic",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinTrafficUsageSeries(**data)

    @classmethod
    async def get_users_stats(
        cls, host: str, token: str, timeout: int = 10
    ) -> MarzneshinUsersStats:
        """
        Get users statistics.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Users stats
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/system/stats/users",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzneshinUsersStats(**data)



from .core import RequestBase
from .marzneshin import MarzneshinManager
from .marzban import MarzbanManager
from .guard import GuardManager
from .pasarguard import PasarGuardManager
from .remnawave import RemnawaveManager
from .ovpanel import OVPanelManager

__all__ = [
    "RequestBase",
    "MarzneshinManager",
    "MarzbanManager",
    "GuardManager",
    "PasarGuardManager",
    "RemnawaveManager",
    "OVPanelManager",
]



from .manager import MarzbanManager
from .types import (
    MarzbanAdmin,
    MarzbanAdminCreate,
    MarzbanAdminModify,
    MarzbanToken,
    MarzbanUserCreate,
    MarzbanUserModify,
    MarzbanUserResponse,
    MarzbanUsersResponse,
    MarzbanUserStatus,
    MarzbanUserStatusCreate,
    MarzbanUserStatusModify,
    MarzbanUserDataLimitResetStrategy,
    MarzbanProxyTypes,
    MarzbanNextPlanModel,
    MarzbanUserUsageResponse,
    MarzbanUserUsagesResponse,
    MarzbanUsersUsagesResponse,
    MarzbanNodeCreate,
    MarzbanNodeModify,
    MarzbanNodeResponse,
    MarzbanNodeSettings,
    MarzbanNodeStatus,
    MarzbanNodeUsageResponse,
    MarzbanNodesUsageResponse,
    MarzbanCoreStats,
    MarzbanSystemStats,
    MarzbanProxyInbound,
    MarzbanProxyHost,
    MarzbanProxyHostSecurity,
    MarzbanProxyHostALPN,
    MarzbanProxyHostFingerprint,
    MarzbanSubscriptionUserResponse,
    MarzbanUserTemplateCreate,
    MarzbanUserTemplateModify,
    MarzbanUserTemplateResponse,
)


__all__ = [
    "MarzbanManager",
    "MarzbanAdmin",
    "MarzbanAdminCreate",
    "MarzbanAdminModify",
    "MarzbanToken",
    "MarzbanUserCreate",
    "MarzbanUserModify",
    "MarzbanUserResponse",
    "MarzbanUsersResponse",
    "MarzbanUserStatus",
    "MarzbanUserStatusCreate",
    "MarzbanUserStatusModify",
    "MarzbanUserDataLimitResetStrategy",
    "MarzbanProxyTypes",
    "MarzbanNextPlanModel",
    "MarzbanUserUsageResponse",
    "MarzbanUserUsagesResponse",
    "MarzbanUsersUsagesResponse",
    "MarzbanNodeCreate",
    "MarzbanNodeModify",
    "MarzbanNodeResponse",
    "MarzbanNodeSettings",
    "MarzbanNodeStatus",
    "MarzbanNodeUsageResponse",
    "MarzbanNodesUsageResponse",
    "MarzbanCoreStats",
    "MarzbanSystemStats",
    "MarzbanProxyInbound",
    "MarzbanProxyHost",
    "MarzbanProxyHostSecurity",
    "MarzbanProxyHostALPN",
    "MarzbanProxyHostFingerprint",
    "MarzbanSubscriptionUserResponse",
    "MarzbanUserTemplateCreate",
    "MarzbanUserTemplateModify",
    "MarzbanUserTemplateResponse",
]



from typing import Optional, Dict
from aiohttp import ClientSession, ClientTimeout, ClientResponse


class RequestBase:
    @classmethod
    async def fetch(
        cls,
        method: str,
        url: str,
        params: Optional[Dict] = None,
        data: Optional[Dict] = None,
        headers: Optional[Dict] = None,
        timeout: int = 10,
    ) -> ClientResponse:
        """
        Perform an HTTP request and return the result.
        :param method: HTTP method (GET, POST, etc.)
        :param url: URL to request
        :param params: Query parameters
        :param data: Request body data
        :param headers: Request headers
        :param timeout: Request timeout in seconds
        :return: ClientResponse containing status and content
        """
        async with ClientSession(timeout=ClientTimeout(total=timeout)) as session:
            async with session.request(
                method=method,
                url=url,
                params=params,
                data=data,
                headers=headers,
            ) as response:
                response.raise_for_status()
                return response

    @classmethod
    async def get(
        cls,
        url: str,
        params: Optional[Dict] = None,
        headers: Optional[Dict] = None,
        timeout: int = 10,
    ) -> ClientResponse:
        """
        Perform a GET request.
        :param url: URL to request
        :param params: Query parameters
        :param headers: Request headers
        :param timeout: Request timeout in seconds
        :return: ClientResponse containing status and content
        """
        return await cls.fetch(
            method="GET",
            url=url,
            params=params,
            headers=headers,
            timeout=timeout,
        )

    @classmethod
    async def post(
        cls,
        url: str,
        params: Optional[Dict] = None,
        data: Optional[Dict] = None,
        headers: Optional[Dict] = None,
        timeout: int = 10,
    ) -> ClientResponse:
        """
        Perform a POST request.
        :param url: URL to request
        :param data: Request body data
        :param headers: Request headers
        :param timeout: Request timeout in seconds
        :return: ClientResponse containing status and content
        """
        return await cls.fetch(
            method="POST",
            url=url,
            data=data,
            params=params,
            headers=headers,
            timeout=timeout,
        )

    @classmethod
    async def delete(
        cls,
        url: str,
        params: Optional[Dict] = None,
        data: Optional[Dict] = None,
        headers: Optional[Dict] = None,
        timeout: int = 10,
    ) -> ClientResponse:
        """
        Perform a DELETE request.
        :param url: URL to request
        :param headers: Request headers
        :param timeout: Request timeout in seconds
        :return: ClientResponse containing status and content
        """
        return await cls.fetch(
            method="DELETE",
            url=url,
            data=data,
            params=params,
            headers=headers,
            timeout=timeout,
        )

    @classmethod
    async def put(
        cls,
        url: str,
        params: Optional[Dict] = None,
        data: Optional[Dict] = None,
        headers: Optional[Dict] = None,
        timeout: int = 10,
    ) -> ClientResponse:
        """
        Perform a PUT request.
        :param url: URL to request
        :param data: Request body data
        :param headers: Request headers
        :param timeout: Request timeout in seconds
        :return: ClientResponse containing status and content
        """
        return await cls.fetch(
            method="PUT",
            url=url,
            data=data,
            params=params,
            headers=headers,
            timeout=timeout,
        )



from typing import Optional, List
from pydantic import BaseModel, Field


class MarzneshinServiceCreate(BaseModel):
    """Schema for creating a new service"""

    name: Optional[str] = Field(None, title="Name")
    inbound_ids: List[int] = Field(default_factory=list, title="Inbound Ids")


class MarzneshinServiceModify(BaseModel):
    """Schema for modifying an existing service"""

    name: Optional[str] = Field(None, title="Name")
    inbound_ids: Optional[List[int]] = Field(None, title="Inbound Ids")


class MarzneshinServiceResponse(BaseModel):
    """Service response schema"""

    id: int = Field(..., title="Id")
    name: Optional[str] = Field(None, title="Name")
    inbound_ids: List[int] = Field(..., title="Inbound Ids")
    user_ids: List[int] = Field(..., title="User Ids")



from typing import Optional, List, Dict, Any
from pydantic import BaseModel, Field


class PasarGuardHostCreate(BaseModel):
    """Schema for creating/modifying a host"""

    id: Optional[int] = Field(None, title="Id")
    remark: str = Field(..., title="Remark")
    address: List[str] = Field(..., title="Address")
    inbound_tag: Optional[str] = Field(None, title="Inbound Tag")
    port: Optional[int] = Field(None, title="Port")
    sni: Optional[List[str]] = Field(None, title="Sni")
    host: Optional[List[str]] = Field(None, title="Host")
    path: Optional[str] = Field(None, title="Path")
    security: str = Field("inbound_default", title="Security")
    alpn: Optional[List[str]] = Field(None, title="Alpn")
    fingerprint: str = Field("", title="Fingerprint")
    allowinsecure: Optional[bool] = Field(None, title="Allowinsecure")
    is_disabled: bool = Field(False, title="Is Disabled")
    http_headers: Optional[Dict[str, str]] = Field(None, title="Http Headers")
    transport_settings: Optional[Dict[str, Any]] = Field(
        None, title="Transport Settings"
    )
    mux_settings: Optional[Dict[str, Any]] = Field(None, title="Mux Settings")
    fragment_settings: Optional[Dict[str, Any]] = Field(None, title="Fragment Settings")
    noise_settings: Optional[Dict[str, Any]] = Field(None, title="Noise Settings")
    random_user_agent: bool = Field(False, title="Random User Agent")
    use_sni_as_host: bool = Field(False, title="Use Sni As Host")
    priority: int = Field(..., title="Priority")
    status: Optional[List[str]] = Field(None, title="Status")
    ech_config_list: Optional[str] = Field(None, title="Ech Config List")


class PasarGuardHostResponse(PasarGuardHostCreate):
    """Host response schema"""

    pass



from typing import Optional
from datetime import datetime
from enum import Enum
from pydantic import BaseModel, Field


class GuardNodeCategory(str, Enum):
    """Node category types"""

    MARZBAN = "marzban"
    MARZNESHIN = "marzneshin"


class GuardNodeCreate(BaseModel):
    """Schema for creating a new node"""

    remark: str = Field(..., title="Remark")
    category: GuardNodeCategory = Field(..., title="Category")
    username: str = Field(..., title="Username")
    password: str = Field(..., title="Password")
    host: str = Field(..., title="Host")
    offset_link: int = Field(0, title="Offset Link")
    batch_size: int = Field(1, title="Batch Size")


class GuardNodeUpdate(BaseModel):
    """Schema for updating an existing node"""

    remark: Optional[str] = Field(None, title="Remark")
    username: Optional[str] = Field(None, title="Username")
    password: Optional[str] = Field(None, title="Password")
    host: Optional[str] = Field(None, title="Host")
    offset_link: Optional[int] = Field(None, title="Offset Link")
    batch_size: Optional[int] = Field(None, title="Batch Size")


class GuardNodeResponse(BaseModel):
    """Node response schema"""

    id: int = Field(..., title="Id")
    enabled: bool = Field(..., title="Enabled")
    remark: str = Field(..., title="Remark")
    category: GuardNodeCategory = Field(..., title="Category")
    username: str = Field(..., title="Username")
    password: str = Field(..., title="Password")
    host: str = Field(..., title="Host")
    current_usage: int = Field(..., title="Current Usage")
    last_used_at: Optional[datetime] = Field(..., title="Last Used At")
    offset_link: int = Field(..., title="Offset Link")
    batch_size: int = Field(..., title="Batch Size")
    created_at: datetime = Field(..., title="Created At")
    updated_at: datetime = Field(..., title="Updated At")


class GuardNodeStatsResponse(BaseModel):
    """Node statistics response"""

    total_nodes: int = Field(..., title="Total Nodes")
    active_nodes: int = Field(..., title="Active Nodes")
    inactive_nodes: int = Field(..., title="Inactive Nodes")



from typing import Optional
from pydantic import BaseModel, Field


class PasarGuardSystemStats(BaseModel):
    """System statistics response schema"""

    version: str = Field(..., title="Version")
    mem_total: Optional[int] = Field(None, title="Mem Total")
    mem_used: Optional[int] = Field(None, title="Mem Used")
    cpu_cores: Optional[int] = Field(None, title="Cpu Cores")
    cpu_usage: Optional[float] = Field(None, title="Cpu Usage")
    total_user: int = Field(..., title="Total User")
    online_users: int = Field(..., title="Online Users")
    active_users: int = Field(..., title="Active Users")
    on_hold_users: int = Field(..., title="On Hold Users")
    disabled_users: int = Field(..., title="Disabled Users")
    expired_users: int = Field(..., title="Expired Users")
    limited_users: int = Field(..., title="Limited Users")
    incoming_bandwidth: int = Field(..., title="Incoming Bandwidth")
    outgoing_bandwidth: int = Field(..., title="Outgoing Bandwidth")



from pydantic import BaseModel, Field


class RemnawaveUserStatistics(BaseModel):
    """User statistics"""

    active: int
    disabled: int
    limited: int
    expired: int

    class Config:
        populate_by_name = True


class RemnawaveOnlineStatistics(BaseModel):
    """Online statistics"""

    last_day: int = Field(..., alias="lastDay")
    last_week: int = Field(..., alias="lastWeek")
    never_online: int = Field(..., alias="neverOnline")
    online_now: int = Field(..., alias="onlineNow")

    class Config:
        populate_by_name = True


class RemnawaveNodeStatistics(BaseModel):
    """Node statistics"""

    total_online: int = Field(..., alias="totalOnline")

    class Config:
        populate_by_name = True


class RemnawaveSystemStats(BaseModel):
    """System statistics response"""

    user_stats: RemnawaveUserStatistics = Field(..., alias="userStats")
    online_stats: RemnawaveOnlineStatistics = Field(..., alias="onlineStats")
    nodes: RemnawaveNodeStatistics

    class Config:
        populate_by_name = True



from typing import Optional, List
from enum import Enum
from pydantic import BaseModel, Field


class MarzneshinNodeStatus(str, Enum):
    """Node status enumeration"""

    HEALTHY = "healthy"
    UNHEALTHY = "unhealthy"
    DISABLED = "disabled"


class MarzneshinNodeConnectionBackend(str, Enum):
    """Node connection backend enumeration"""

    GRPCIO = "grpcio"
    GRPCLIB = "grpclib"


class MarzneshinNodeCreate(BaseModel):
    """Schema for creating a new node"""

    name: str = Field(..., title="Name")
    address: str = Field(..., title="Address")
    port: int = Field(53042, title="Port")
    connection_backend: MarzneshinNodeConnectionBackend = Field(
        MarzneshinNodeConnectionBackend.GRPCLIB, title="Connection Backend"
    )
    usage_coefficient: float = Field(1.0, ge=0.0, title="Usage Coefficient")


class MarzneshinNodeModify(BaseModel):
    """Schema for modifying an existing node"""

    name: Optional[str] = Field(None, title="Name")
    address: Optional[str] = Field(None, title="Address")
    port: Optional[int] = Field(None, title="Port")
    connection_backend: Optional[MarzneshinNodeConnectionBackend] = Field(
        None, title="Connection Backend"
    )
    usage_coefficient: Optional[float] = Field(None, ge=0.0, title="Usage Coefficient")
    status: Optional[MarzneshinNodeStatus] = Field(None, title="Status")


class MarzneshinBackend(BaseModel):
    """Backend information"""

    name: str = Field(..., title="Name")
    backend_type: str = Field(..., title="Backend Type")
    version: Optional[str] = Field(None, title="Version")
    running: bool = Field(..., title="Running")


class MarzneshinNodeResponse(BaseModel):
    """Node response schema"""

    id: int = Field(..., title="Id")
    name: str = Field(..., title="Name")
    address: str = Field(..., title="Address")
    port: int = Field(53042, title="Port")
    connection_backend: MarzneshinNodeConnectionBackend = Field(
        MarzneshinNodeConnectionBackend.GRPCLIB, title="Connection Backend"
    )
    usage_coefficient: float = Field(1.0, ge=0.0, title="Usage Coefficient")
    xray_version: Optional[str] = Field(None, title="Xray Version")
    status: MarzneshinNodeStatus = Field(..., title="Status")
    message: Optional[str] = Field(None, title="Message")
    inbound_ids: Optional[List[int]] = Field(None, title="Inbound Ids")
    backends: List[MarzneshinBackend] = Field(..., title="Backends")


class MarzneshinNodeSettings(BaseModel):
    """Node settings response"""

    min_node_version: str = Field("v0.2.0", title="Min Node Version")
    certificate: str = Field(..., title="Certificate")



from pydantic import BaseModel, Field


class OVPanelToken(BaseModel):
    """Authentication token response"""

    access_token: str = Field(..., title="Access Token")
    token_type: str = Field("bearer", title="Token Type")


class OVPanelAdmin(BaseModel):
    """Admin response schema"""

    username: str = Field(..., title="Username")
    is_sudo: bool = Field(..., title="Is Sudo")



from typing import Optional, List
from datetime import datetime
from enum import Enum
from pydantic import BaseModel, Field


class MarzneshinUserExpireStrategy(str, Enum):
    """User expire strategy enumeration"""

    NEVER = "never"
    FIXED_DATE = "fixed_date"
    START_ON_FIRST_USE = "start_on_first_use"


class MarzneshinUserDataUsageResetStrategy(str, Enum):
    """User data usage reset strategy enumeration"""

    NO_RESET = "no_reset"
    DAY = "day"
    WEEK = "week"
    MONTH = "month"
    YEAR = "year"


class MarzneshinUsersSortingOptions(str, Enum):
    """Users sorting options enumeration"""

    USERNAME = "username"
    USED_TRAFFIC = "used_traffic"
    DATA_LIMIT = "data_limit"
    EXPIRE_DATE = "expire_date"
    CREATED_AT = "created_at"


class MarzneshinUserCreate(BaseModel):
    """Schema for creating a new user"""

    username: str = Field(..., pattern=r"^\w{3,32}$", title="Username")
    expire_strategy: MarzneshinUserExpireStrategy = Field(..., title="Expire Strategy")
    expire_date: Optional[datetime] = Field(None, title="Expire Date")
    usage_duration: Optional[int] = Field(None, title="Usage Duration")
    activation_deadline: Optional[datetime] = Field(None, title="Activation Deadline")
    data_limit: Optional[int] = Field(
        None, ge=0, title="Data Limit", description="data_limit can be 0 or greater"
    )
    data_limit_reset_strategy: MarzneshinUserDataUsageResetStrategy = Field(
        MarzneshinUserDataUsageResetStrategy.NO_RESET, title="Data Limit Reset Strategy"
    )
    note: Optional[str] = Field(None, max_length=500, title="Note")
    service_ids: List[int] = Field(default_factory=list, title="Service Ids")


class MarzneshinUserModify(BaseModel):
    """Schema for modifying an existing user"""

    username: str = Field(..., pattern=r"^\w{3,32}$", title="Username")
    expire_strategy: Optional[MarzneshinUserExpireStrategy] = Field(
        None, title="Expire Strategy"
    )
    expire_date: Optional[datetime] = Field(None, title="Expire Date")
    usage_duration: Optional[int] = Field(None, title="Usage Duration")
    activation_deadline: Optional[datetime] = Field(None, title="Activation Deadline")
    data_limit: Optional[int] = Field(
        None, ge=0, title="Data Limit", description="data_limit can be 0 or greater"
    )
    data_limit_reset_strategy: Optional[MarzneshinUserDataUsageResetStrategy] = Field(
        None, title="Data Limit Reset Strategy"
    )
    note: Optional[str] = Field(None, max_length=500, title="Note")
    service_ids: Optional[List[int]] = Field(None, title="Service Ids")


class MarzneshinUserResponse(BaseModel):
    """User response schema"""

    id: int = Field(..., title="Id")
    username: str = Field(..., pattern=r"^\w{3,32}$", title="Username")
    expire_strategy: MarzneshinUserExpireStrategy = Field(..., title="Expire Strategy")
    expire_date: Optional[datetime] = Field(None, title="Expire Date")
    usage_duration: Optional[int] = Field(None, title="Usage Duration")
    activation_deadline: Optional[datetime] = Field(None, title="Activation Deadline")
    key: str = Field(..., title="Key")
    data_limit: Optional[int] = Field(
        None, ge=0, title="Data Limit", description="data_limit can be 0 or greater"
    )
    data_limit_reset_strategy: MarzneshinUserDataUsageResetStrategy = Field(
        MarzneshinUserDataUsageResetStrategy.NO_RESET, title="Data Limit Reset Strategy"
    )
    note: Optional[str] = Field(None, max_length=500, title="Note")
    activated: bool = Field(..., title="Activated")
    is_active: bool = Field(..., title="Is Active")
    expired: bool = Field(..., title="Expired")
    data_limit_reached: bool = Field(..., title="Data Limit Reached")
    enabled: bool = Field(..., title="Enabled")
    used_traffic: int = Field(..., title="Used Traffic")
    lifetime_used_traffic: int = Field(..., title="Lifetime Used Traffic")
    sub_revoked_at: Optional[datetime] = Field(None, title="Sub Revoked At")
    created_at: datetime = Field(..., title="Created At")
    service_ids: List[int] = Field(..., title="Service Ids")
    subscription_url: str = Field(..., title="Subscription Url")
    owner_username: Optional[str] = Field(None, title="Owner Username")
    traffic_reset_at: Optional[datetime] = Field(None, title="Traffic Reset At")
    sub_updated_at: Optional[datetime] = Field(None, title="Sub Updated At")
    sub_last_user_agent: Optional[str] = Field(None, title="Sub Last User Agent")
    online_at: Optional[datetime] = Field(None, title="Online At")


class MarzneshinUserNodeUsageSeries(BaseModel):
    """User node usage series"""

    node_id: Optional[int] = Field(None, title="Node Id")
    node_name: str = Field(..., title="Node Name")
    usages: List[List[int]] = Field(..., title="Usages")


class MarzneshinUserUsageSeriesResponse(BaseModel):
    """User usage series response"""

    username: str = Field(..., title="Username")
    node_usages: List[MarzneshinUserNodeUsageSeries] = Field(..., title="Node Usages")
    total: int = Field(..., title="Total")



from typing import Optional, Any
from datetime import date
from pydantic import BaseModel, Field


class OVPanelCreateUser(BaseModel):
    """Schema for creating a new user"""

    name: str = Field(..., min_length=3, max_length=10, title="Name")
    expiry_date: date = Field(..., title="Expiry Date")


class OVPanelUpdateUser(BaseModel):
    """Schema for updating an existing user"""

    name: str = Field(..., title="Name")
    expiry_date: Optional[date] = Field(None, title="Expiry Date")
    status: bool = Field(True, title="Status")


class OVPanelUser(BaseModel):
    """User response schema"""

    name: str = Field(..., title="Name")
    expiry_date: date = Field(..., title="Expiry Date")
    status: bool = Field(..., title="Status")


class OVPanelResponseModel(BaseModel):
    """Generic response model"""

    success: bool = Field(..., title="Success")
    msg: str = Field(..., title="Message")
    data: Optional[Any] = Field(None, title="Data")



from typing import Optional
from pydantic import BaseModel, Field


class OVPanelNodeCreate(BaseModel):
    """Schema for creating a new node"""

    name: str = Field(..., max_length=10, title="Name")
    address: str = Field(..., title="Address")
    tunnel_address: Optional[str] = Field(None, title="Tunnel Address")
    protocol: str = Field("tcp", title="Protocol")
    ovpn_port: int = Field(1194, title="OVPN Port")
    port: int = Field(..., title="Port")
    key: str = Field(..., min_length=10, max_length=40, title="Key")
    status: bool = Field(True, title="Status")
    set_new_setting: bool = Field(False, title="Set New Setting")


class OVPanelNodeStatus(BaseModel):
    """Node status response schema"""

    success: bool = Field(..., title="Success")
    msg: str = Field(..., title="Message")
    data: Optional[dict] = Field(None, title="Data")


class OVPanelNode(BaseModel):
    """Node response schema"""

    name: str = Field(..., title="Name")
    address: str = Field(..., title="Address")
    tunnel_address: Optional[str] = Field(None, title="Tunnel Address")
    protocol: str = Field(..., title="Protocol")
    ovpn_port: int = Field(..., title="OVPN Port")
    port: int = Field(..., title="Port")
    key: str = Field(..., title="Key")
    status: bool = Field(..., title="Status")
